import java.awt.event.*;
import java.util.*;
import org.junit.*;
import org.mockito.invocation.*;
import org.mockito.stubbing.*;
import static org.mockito.Mockito.*;

//java -cp junit-4.11.jar:mockito-all-1.9.5.jar:. org.junit.runner.JUnitCore LoginPresenterTest
public class LoginPresenterTest{
	@Test
	public void init(){
		ILoginView view = mock(ILoginView.class);

		ILoginModel model = mock(ILoginModel.class);
		when(model.getCachedUsername()).thenReturn("user");
		when(model.getCachedPassword()).thenReturn("password");
		when(model.getCachedRememberMe()).thenReturn(true);

		LoginPresenter presenter = new LoginPresenter(view, model);

		verify(view).addLoginListener(any(ActionListener.class));
		verify(view).addCancelListener(any(ActionListener.class));
		verify(view).setUsername("user");
		verify(view).setPassword("password");
		verify(view).setRememberMe(true);
		verify(view).display();
	}

	@Test
	public void bad_login(){
		ILoginView view = mock(ILoginView.class);
		when(view.getUsername()).thenReturn("user");
		when(view.getPassword()).thenReturn("password");
		ListenerAnswer loginAnswer = new ListenerAnswer();
		doAnswer(loginAnswer).when(view).addLoginListener(any(ActionListener.class));

		ILoginModel model = mock(ILoginModel.class);
		when(model.login("user", "password")).thenReturn(null); //"null" = bad login

		LoginPresenter presenter = new LoginPresenter(view, model);

		//click "login"
		loginAnswer.fire();

		verify(model, never()).setSession(anyString());
		verify(view, never()).onSuccessfulLogin();
		verify(view).onBadLogin();
		verify(view, never()).close();
	}

	@Test
	public void valid_login(){
		ILoginView view = mock(ILoginView.class);
		when(view.getUsername()).thenReturn("user");
		when(view.getPassword()).thenReturn("password");
		ListenerAnswer loginAnswer = new ListenerAnswer();
		doAnswer(loginAnswer).when(view).addLoginListener(any(ActionListener.class));

		ILoginModel model = mock(ILoginModel.class);
		when(model.login("user", "password")).thenReturn("abc123"); //non-null token = good login

		LoginPresenter presenter = new LoginPresenter(view, model);

		//click "login"
		loginAnswer.fire();

		verify(model).setSession("abc123");
		verify(view, never()).onBadLogin();
		verify(view).onSuccessfulLogin();
		verify(view).close();
	}

	@Test
	public void rememberMe_true(){
		ILoginView view = mock(ILoginView.class);
		when(view.getUsername()).thenReturn("user");
		when(view.getPassword()).thenReturn("password");
		when(view.getRememberMe()).thenReturn(true);
		ListenerAnswer loginAnswer = new ListenerAnswer();
		doAnswer(loginAnswer).when(view).addLoginListener(any(ActionListener.class));

		ILoginModel model = mock(ILoginModel.class);
		when(model.login("user", "password")).thenReturn("abc123");

		LoginPresenter presenter = new LoginPresenter(view, model);

		//click "login"
		loginAnswer.fire();

		verify(model).setCachedUsername("user");
		verify(model).setCachedPassword("password");
		verify(model).setCachedRememberMe(true);
	}

	@Test
	public void rememberMe_false(){
		ILoginView view = mock(ILoginView.class);
		when(view.getUsername()).thenReturn("user");
		when(view.getPassword()).thenReturn("password");
		when(view.getRememberMe()).thenReturn(false);
		ListenerAnswer loginAnswer = new ListenerAnswer();
		doAnswer(loginAnswer).when(view).addLoginListener(any(ActionListener.class));

		ILoginModel model = mock(ILoginModel.class);
		when(model.login("user", "password")).thenReturn("abc123");

		LoginPresenter presenter = new LoginPresenter(view, model);

		//click "login"
		loginAnswer.fire();

		verify(model).setCachedUsername("");
		verify(model).setCachedPassword("");
		verify(model).setCachedRememberMe(false);
	}

	@Test
	public void cancel(){
		ILoginView view = mock(ILoginView.class);
		ListenerAnswer cancelAnswer = new ListenerAnswer();
		doAnswer(cancelAnswer).when(view).addCancelListener(any(ActionListener.class));

		ILoginModel model = mock(ILoginModel.class);

		LoginPresenter presenter = new LoginPresenter(view, model);

		//click "cancel"
		cancelAnswer.fire();

		verify(model, never()).setSession(anyString());
		verify(view).close();
	}

	private class ListenerAnswer implements Answer<Object>{
		private final List<ActionListener> listeners = new ArrayList<ActionListener>();

		public Object answer(InvocationOnMock invocation) {
			ActionListener listener = (ActionListener)invocation.getArguments()[0];
			listeners.add(listener);
			return null;
		}

		public void fire(){
			for (ActionListener listener : listeners){
				listener.actionPerformed(null);
			}
		}
	}
}