import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import static javax.swing.SpringLayout.*;

public class LoginViewImpl extends JFrame implements ILoginView {
	private final JButton login, cancel;
	private final JTextField username;
	private final JPasswordField password;
	private final JCheckBox rememberMe;

	public LoginViewImpl() {
		setTitle("Login");
		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);

		login = new JButton("Login");
		cancel = new JButton("Cancel");
		username = new JTextField();
		password = new JPasswordField();
		rememberMe = new JCheckBox("Remember me");

		JLabel title = new JLabel("Please enter your login credentials.");
		JLabel usernameLabel = new JLabel("Username:");
		JLabel passwordLabel = new JLabel("Password:");

		///////////////////////

		Container contentPane = getContentPane();
		SpringLayout layout = new SpringLayout();
		contentPane.setLayout(layout);

		contentPane.add(title);
		contentPane.add(usernameLabel);
		contentPane.add(username);
		contentPane.add(passwordLabel);
		contentPane.add(password);
		contentPane.add(rememberMe);
		contentPane.add(login);
		contentPane.add(cancel);

		layout.putConstraint(WEST, title, 5, WEST, contentPane);
		layout.putConstraint(NORTH, title, 5, NORTH, contentPane);

		layout.putConstraint(WEST, usernameLabel, 5, WEST, contentPane);
		layout.putConstraint(NORTH, usernameLabel, 10, SOUTH, title);
		layout.putConstraint(WEST, username, 10, EAST, usernameLabel);
		layout.putConstraint(NORTH, username, 0, NORTH, usernameLabel);
		layout.putConstraint(EAST, username, 100, WEST, username);

		layout.putConstraint(WEST, passwordLabel, 5, WEST, contentPane);
		layout.putConstraint(NORTH, passwordLabel, 5, SOUTH, usernameLabel);
		layout.putConstraint(WEST, password, 0, WEST, username);
		layout.putConstraint(NORTH, password, 0, NORTH, passwordLabel);
		layout.putConstraint(EAST, password, 100, WEST, password);

		layout.putConstraint(WEST, rememberMe, 5, WEST, contentPane);
		layout.putConstraint(NORTH, rememberMe, 5, SOUTH, passwordLabel);

		layout.putConstraint(WEST, login, 5, WEST, contentPane);
		layout.putConstraint(NORTH, login, 10, SOUTH, rememberMe);
		layout.putConstraint(WEST, cancel, 5, EAST, login);
		layout.putConstraint(NORTH, cancel, 0, NORTH, login);

		setSize(300,200);
		setLocationRelativeTo(null);
	}

	public void addLoginListener(ActionListener listener) {
		login.addActionListener(listener);
		username.addActionListener(listener);
		password.addActionListener(listener);
	}

	public void addCancelListener(final ActionListener listener) {
		cancel.addActionListener(listener);
		addWindowListener(new WindowAdapter(){
			public void windowClosing(WindowEvent event){
				listener.actionPerformed(null);
			}
		});
	}

	public String getUsername() {
		return username.getText();
	}

	public void setUsername(String username) {
		this.username.setText(username);
	}

	public String getPassword() {
		return new String(password.getPassword());
	}

	public void setPassword(String password){
		this.password.setText(password);
	}

	public boolean getRememberMe() {
		return rememberMe.isSelected();
	}

	public void setRememberMe(boolean rememberMe) {
		this.rememberMe.setSelected(rememberMe);
	}

	public void onBadLogin() {
		JOptionPane.showMessageDialog(this, "Invalid login credentials.");
	}

	public void onSuccessfulLogin() {
		JOptionPane.showMessageDialog(this, "Login successful.");
	}

	public void display() {
		setVisible(true);
	}

	public void close() {
		dispose();
	}
}
